@@ -56,18 +56,16 @@ header_portindexed_table(struct variable *vp, oid *name, size_t *length,
 	port = name[*length - 1];
 	distance = -1;
 	TAILQ_FOREACH(hardware, &scfg->g_hardware, h_entries) {
-		if (INTERFACE_OPENED(hardware)) {
-			aport = if_nametoindex(hardware->h_ifname);
-			if (aport == port) {
-                                /* Exact match */
-                                return hardware;
-			}
-			if (aport < port)
-				continue;
-			if (aport - port < distance) {
-				phardware = hardware;
-				distance = aport - port;
-			}
+		aport = hardware->h_ifindex;
+		if (aport == port) {
+			/* Exact match */
+			return hardware;
+		}
+		if (aport < port)
+			continue;
+		if (aport - port < distance) {
+			phardware = hardware;
+			distance = aport - port;
 		}
 	}
 	if (phardware == NULL)
@@ -81,15 +79,83 @@ header_portindexed_table(struct variable *vp, oid *name, size_t *length,
 	return phardware;
 }
 
+#ifdef ENABLE_LLDPMED
+#define PMED_VARIANT_POLICY 0
+#define PMED_VARIANT_LOCATION   1
+static void*
+header_pmedindexed_table(struct variable *vp, oid *name, size_t *length,
+    int exact, size_t *var_len, WriteMethod **write_method, int variant)
+{
+	struct lldpd_hardware *hardware;
+	void *squid, *psquid = NULL;
+	int result, target_len, i, max;
+        oid *target, current[2], best[2];
+
+        if ((result = snmp_oid_compare(name, *length, vp->name, vp->namelen)) < 0) {
+                memcpy(name, vp->name, sizeof(oid) * vp->namelen);
+                *length = vp->namelen;
+        }
+
+	*write_method = 0;
+	*var_len = sizeof(long);
+
+	best[0] = best[1] = MAX_SUBID;
+        target = &name[vp->namelen];
+        target_len = *length - vp->namelen;
+	TAILQ_FOREACH(hardware, &scfg->g_hardware, h_entries) {
+		max = (variant == PMED_VARIANT_POLICY)?
+		    LLDPMED_APPTYPE_LAST:LLDPMED_LOCFORMAT_LAST;
+		for (i = 0;
+		     i < max;
+		     i++) {
+			if (variant == PMED_VARIANT_POLICY) {
+				if (hardware->h_lport.p_med_policy[i].type != i+1)
+					continue;
+				squid = &(hardware->h_lport.p_med_policy[i]);
+			} else {
+				if (hardware->h_lport.p_med_location[i].format != i+1)
+					continue;
+				squid = &(hardware->h_lport.p_med_location[i]);
+			}
+			current[0] = hardware->h_ifindex;
+			current[1] = i+1;
+			if ((result = snmp_oid_compare(current, 2, target,
+				    target_len)) < 0)
+				continue;
+			if ((result == 0) && !exact)
+				continue;
+			if (result == 0)
+				return squid;
+			if (snmp_oid_compare(current, 2, best, 2) < 0) {
+				memcpy(best, current, sizeof(oid) * 2);
+				psquid = squid;
+			}
+		}
+	}
+	if (psquid == NULL)
+		return NULL;
+	if (exact)
+		return NULL;
+	if ((best[0] == best[1]) &&
+	    (best[0] == MAX_SUBID))
+		return NULL;
+	memcpy(target, best, sizeof(oid) * 2);
+	*length = vp->namelen + 2;
+
+	return psquid;
+}
+#endif
+
 #define TPR_VARIANT_NONE 0
 #define TPR_VARIANT_IP   1
 #define TPR_VARIANT_MED_POLICY 2
 #define TPR_VARIANT_MED_LOCATION 3
-static struct lldpd_hardware*
+static struct lldpd_port*
 header_tprindexed_table(struct variable *vp, oid *name, size_t *length,
     int exact, size_t *var_len, WriteMethod **write_method, int variant)
 {
-	struct lldpd_hardware *hardware, *phardware = NULL;
+	struct lldpd_hardware *hardware = NULL;
+	struct lldpd_port *port, *pport = NULL;
         oid *target, current[9], best[9];
         int result, target_len, oid_len;
         int i, j, k;
@@ -118,28 +184,29 @@ header_tprindexed_table(struct variable *vp, oid *name, size_t *length,
         target = &name[vp->namelen];
         target_len = *length - vp->namelen;
 	TAILQ_FOREACH(hardware, &scfg->g_hardware, h_entries) {
-		if ((INTERFACE_OPENED(hardware)) && (hardware->h_rchassis != NULL)) {
+		TAILQ_FOREACH(port, &hardware->h_rports, p_entries) {
+			if (SMART_HIDDEN(scfg, port)) continue;
                         if ((variant == TPR_VARIANT_IP) &&
-                            (hardware->h_rchassis->c_mgmt.s_addr == INADDR_ANY))
+                            (port->p_chassis->c_mgmt.s_addr == INADDR_ANY))
                                 continue;
-			if (hardware->h_rlastchange > starttime.tv_sec)
+			if (port->p_lastchange > starttime.tv_sec)
 				current[0] =
-				    (hardware->h_rlastchange - starttime.tv_sec)*100;
+				    (port->p_lastchange - starttime.tv_sec)*100;
 			else
 				current[0] = 0;
-                        current[1] = if_nametoindex(hardware->h_ifname);
-                        current[2] = hardware->h_rid;
+                        current[1] = hardware->h_ifindex;
+                        current[2] = port->p_chassis->c_index;
 			k = j = 0;
 			switch (variant) {
 			case TPR_VARIANT_IP:
 				current[3] = 1;
 				current[4] = 4;
-				current[8] = hardware->h_rchassis->c_mgmt.s_addr >> 24;
-				current[7] = (hardware->h_rchassis->c_mgmt.s_addr &
+				current[8] = port->p_chassis->c_mgmt.s_addr >> 24;
+				current[7] = (port->p_chassis->c_mgmt.s_addr &
 				    0xffffff) >> 16;
-				current[6] = (hardware->h_rchassis->c_mgmt.s_addr &
+				current[6] = (port->p_chassis->c_mgmt.s_addr &
 				    0xffff) >> 8;
-				current[5] = hardware->h_rchassis->c_mgmt.s_addr &
+				current[5] = port->p_chassis->c_mgmt.s_addr &
 				    0xff;
 				break;
 #ifdef ENABLE_LLDPMED
@@ -160,15 +227,15 @@ header_tprindexed_table(struct variable *vp, oid *name, size_t *length,
 				if ((result == 0) && !exact)
 					continue;
 				if (result == 0)
-					return hardware;
+					return port;
 				if (snmp_oid_compare(current, oid_len, best, oid_len) < 0) {
 					memcpy(best, current, sizeof(oid) * oid_len);
-					phardware = hardware;
+					pport = port;
 				}
 			} while (k < j);
 		}
 	}
-	if (phardware == NULL)
+	if (pport == NULL)
 		return NULL;
 	if (exact)
 		return NULL;
@@ -179,7 +246,7 @@ header_tprindexed_table(struct variable *vp, oid *name, size_t *length,
         memcpy(target, best, sizeof(oid) * oid_len);
         *length = vp->namelen + oid_len;
 
-	return phardware;
+	return pport;
 }
 
 #ifdef ENABLE_DOT1
@@ -204,22 +271,20 @@ header_pvindexed_table(struct variable *vp, oid *name, size_t *length,
         target = &name[vp->namelen];
         target_len = *length - vp->namelen;
 	TAILQ_FOREACH(hardware, &scfg->g_hardware, h_entries) {
-		if (INTERFACE_OPENED(hardware)) {
-                        TAILQ_FOREACH(vlan, &hardware->h_lport.p_vlans, v_entries) {
-                                current[0] = if_nametoindex(hardware->h_ifname);
-                                current[1] = vlan->v_vid;
-                                if ((result = snmp_oid_compare(current, 2, target,
-                                            target_len)) < 0)
-                                        continue;
-                                if ((result == 0) && !exact)
-                                        continue;
-                                if (result == 0)
-                                        return vlan;
-                                if (snmp_oid_compare(current, 2, best, 2) < 0) {
-                                        memcpy(best, current, sizeof(oid) * 2);
-                                        pvlan = vlan;
-                                }
-                        }
+		TAILQ_FOREACH(vlan, &hardware->h_lport.p_vlans, v_entries) {
+			current[0] = hardware->h_ifindex;
+			current[1] = vlan->v_vid;
+			if ((result = snmp_oid_compare(current, 2, target,
+				    target_len)) < 0)
+				continue;
+			if ((result == 0) && !exact)
+				continue;
+			if (result == 0)
+				return vlan;
+			if (snmp_oid_compare(current, 2, best, 2) < 0) {
+				memcpy(best, current, sizeof(oid) * 2);
+				pvlan = vlan;
+			}
 		}
 	}
 	if (pvlan == NULL)
@@ -240,6 +305,7 @@ header_tprvindexed_table(struct variable *vp, oid *name, size_t *length,
     int exact, size_t *var_len, WriteMethod **write_method)
 {
 	struct lldpd_hardware *hardware;
+	struct lldpd_port *port;
         struct lldpd_vlan *vlan, *pvlan = NULL;
         oid *target, current[4], best[4];
         int result, target_len;
@@ -256,15 +322,16 @@ header_tprvindexed_table(struct variable *vp, oid *name, size_t *length,
         target = &name[vp->namelen];
         target_len = *length - vp->namelen;
 	TAILQ_FOREACH(hardware, &scfg->g_hardware, h_entries) {
-		if ((INTERFACE_OPENED(hardware)) && (hardware->h_rport != NULL)) {
-                        TAILQ_FOREACH(vlan, &hardware->h_rport->p_vlans, v_entries) {
-				if (hardware->h_rlastchange > starttime.tv_sec)
+		TAILQ_FOREACH(port, &hardware->h_rports, p_entries) {
+			if (SMART_HIDDEN(scfg, port)) continue;
+                        TAILQ_FOREACH(vlan, &port->p_vlans, v_entries) {
+				if (port->p_lastchange > starttime.tv_sec)
 					current[0] =
-					    (hardware->h_rlastchange - starttime.tv_sec)*100;
+					    (port->p_lastchange - starttime.tv_sec)*100;
 				else
 					current[0] = 0;
-                                current[1] = if_nametoindex(hardware->h_ifname);
-                                current[2] = hardware->h_rid;
+                                current[1] = hardware->h_ifindex;
+                                current[2] = port->p_chassis->c_index;
                                 current[3] = vlan->v_vid;
                                 if ((result = snmp_oid_compare(current, 4, target,
                                             target_len)) < 0)
@@ -374,6 +441,13 @@ header_tprvindexed_table(struct variable *vp, oid *name, size_t *length,
 #define LLDP_SNMP_MED_LOCAL_MANUF 6
 #define LLDP_SNMP_MED_LOCAL_MODEL 7
 #define LLDP_SNMP_MED_LOCAL_ASSET 8
+#define LLDP_SNMP_MED_LOCAL_POLICY_VID 9
+#define LLDP_SNMP_MED_LOCAL_POLICY_PRIO 10
+#define LLDP_SNMP_MED_LOCAL_POLICY_DSCP 11
+#define LLDP_SNMP_MED_LOCAL_POLICY_UNKNOWN 12
+#define LLDP_SNMP_MED_LOCAL_POLICY_TAGGED 13
+#define LLDP_SNMP_MED_LOCAL_LOCATION 14
+/* No more than 17 since we reuse LLDP_SNMP_MED_POE_DEVICETYPE and above */
 /* LLDP-MED remote */
 #define LLDP_SNMP_MED_REMOTE_CAP_AVAILABLE 1
 #define LLDP_SNMP_MED_REMOTE_CAP_ENABLED 2
@@ -391,13 +465,13 @@ header_tprvindexed_table(struct variable *vp, oid *name, size_t *length,
 #define LLDP_SNMP_MED_REMOTE_POLICY_UNKNOWN 14
 #define LLDP_SNMP_MED_REMOTE_POLICY_TAGGED 15
 #define LLDP_SNMP_MED_REMOTE_LOCATION 16
-#define LLDP_SNMP_MED_REMOTE_POE_DEVICETYPE 17
-#define LLDP_SNMP_MED_REMOTE_POE_PSE_POWERVAL 19
-#define LLDP_SNMP_MED_REMOTE_POE_PSE_POWERSOURCE 20
-#define LLDP_SNMP_MED_REMOTE_POE_PSE_POWERPRIORITY 21
-#define LLDP_SNMP_MED_REMOTE_POE_PD_POWERVAL 22
-#define LLDP_SNMP_MED_REMOTE_POE_PD_POWERSOURCE 23
-#define LLDP_SNMP_MED_REMOTE_POE_PD_POWERPRIORITY 24
+#define LLDP_SNMP_MED_POE_DEVICETYPE 17
+#define LLDP_SNMP_MED_POE_PSE_POWERVAL 19
+#define LLDP_SNMP_MED_POE_PSE_POWERSOURCE 20
+#define LLDP_SNMP_MED_POE_PSE_POWERPRIORITY 21
+#define LLDP_SNMP_MED_POE_PD_POWERVAL 22
+#define LLDP_SNMP_MED_POE_PD_POWERSOURCE 23
+#define LLDP_SNMP_MED_POE_PD_POWERPRIORITY 24
 
 static u_char*
 agent_h_scalars(struct variable *vp, oid *name, size_t *length,
@@ -405,6 +479,7 @@ agent_h_scalars(struct variable *vp, oid *name, size_t *length,
 {
 	static unsigned long long_ret;
 	struct lldpd_hardware *hardware;
+	struct lldpd_port *port;
 
 	if (header_generic(vp, name, length, exact, var_len, write_method))
 		return NULL;
@@ -414,7 +489,7 @@ agent_h_scalars(struct variable *vp, oid *name, size_t *length,
                 long_ret = scfg->g_delay;
 		return (u_char *)&long_ret;
 	case LLDP_SNMP_TXMULTIPLIER:
-		long_ret = scfg->g_lchassis.c_ttl / scfg->g_delay;
+		long_ret = LOCAL_CHASSIS(scfg)->c_ttl / scfg->g_delay;
 		return (u_char *)&long_ret;
 	case LLDP_SNMP_REINITDELAY:
 		long_ret = 1;
@@ -428,8 +503,11 @@ agent_h_scalars(struct variable *vp, oid *name, size_t *length,
 	case LLDP_SNMP_LASTUPDATE:
 		long_ret = 0;
 		TAILQ_FOREACH(hardware, &scfg->g_hardware, h_entries)
-			if (hardware->h_rlastchange > long_ret)
-				long_ret = hardware->h_rlastchange;
+		    TAILQ_FOREACH(port, &hardware->h_rports, p_entries) {
+			if (SMART_HIDDEN(scfg, port)) continue;
+			if (port->p_lastchange > long_ret)
+				long_ret = port->p_lastchange;
+		}
 		if (long_ret)
 			long_ret = (long_ret - starttime.tv_sec) * 100;
 		return (u_char *)&long_ret;
@@ -461,13 +539,102 @@ agent_h_scalars(struct variable *vp, oid *name, size_t *length,
 }
 
 #ifdef ENABLE_LLDPMED
+static unsigned long
+agent_h_med_power(struct variable *vp, struct lldpd_med_power *power)
+{
+	unsigned long long_ret;
+
+	switch (vp->magic) {
+	case LLDP_SNMP_MED_POE_DEVICETYPE:
+		switch (power->devicetype) {
+		case LLDPMED_POW_TYPE_PSE:
+			long_ret = 2; break;
+		case LLDPMED_POW_TYPE_PD:
+			long_ret = 3; break;
+		case 0:
+			long_ret = 4; break;
+		default:
+			long_ret = 1;
+		}
+		return long_ret;
+	case LLDP_SNMP_MED_POE_PSE_POWERVAL:
+	case LLDP_SNMP_MED_POE_PD_POWERVAL:
+		if (((vp->magic == LLDP_SNMP_MED_POE_PSE_POWERVAL) &&
+			(power->devicetype ==
+			LLDPMED_POW_TYPE_PSE)) ||
+		    ((vp->magic == LLDP_SNMP_MED_POE_PD_POWERVAL) &&
+			(power->devicetype ==
+			    LLDPMED_POW_TYPE_PD))) {
+			long_ret = power->val;
+			return long_ret;
+		}
+		break;
+	case LLDP_SNMP_MED_POE_PSE_POWERSOURCE:
+		if (power->devicetype ==
+		    LLDPMED_POW_TYPE_PSE) {
+			switch (power->source) {
+			case LLDPMED_POW_SOURCE_PRIMARY:
+				long_ret = 2; break;
+			case LLDPMED_POW_SOURCE_BACKUP:
+				long_ret = 3; break;
+			default:
+				long_ret = 1;
+			}
+			return long_ret;
+		}
+		break;
+	case LLDP_SNMP_MED_POE_PD_POWERSOURCE:
+		if (power->devicetype ==
+		    LLDPMED_POW_TYPE_PD) {
+			switch (power->source) {
+			case LLDPMED_POW_SOURCE_PSE:
+				long_ret = 2; break;
+			case LLDPMED_POW_SOURCE_LOCAL:
+				long_ret = 3; break;
+			case LLDPMED_POW_SOURCE_BOTH:
+				long_ret = 4; break;
+			default:
+				long_ret = 1;
+			}
+			return long_ret;
+		}
+		break;
+	case LLDP_SNMP_MED_POE_PSE_POWERPRIORITY:
+	case LLDP_SNMP_MED_POE_PD_POWERPRIORITY:
+		if (((vp->magic == LLDP_SNMP_MED_POE_PSE_POWERPRIORITY) &&
+			(power->devicetype ==
+			LLDPMED_POW_TYPE_PSE)) ||
+		    ((vp->magic == LLDP_SNMP_MED_POE_PD_POWERPRIORITY) &&
+			(power->devicetype ==
+			    LLDPMED_POW_TYPE_PD))) {
+			switch (power->priority) {
+			case LLDPMED_POW_PRIO_CRITICAL:
+				long_ret = 2; break;
+			case LLDPMED_POW_PRIO_HIGH:
+				long_ret = 3; break;
+			case LLDPMED_POW_PRIO_LOW:
+				long_ret = 4; break;
+			default:
+				long_ret = 1;
+			}
+			return long_ret;
+		}
+		break;
+	}
+
+	return (unsigned long)-1;
+}
+
 static u_char*
 agent_h_local_med(struct variable *vp, oid *name, size_t *length,
     int exact, size_t *var_len, WriteMethod **write_method)
 {
         static unsigned long long_ret;
+	struct lldpd_hardware *hardware;
+	struct lldpd_med_power *power;
+	int pse;
 
-	if (!scfg->g_lchassis.c_med_cap_available)
+	if (!LOCAL_CHASSIS(scfg)->c_med_cap_available)
 		return NULL;
 
 	if (header_generic(vp, name, length, exact, var_len, write_method))
@@ -475,15 +642,45 @@ agent_h_local_med(struct variable *vp, oid *name, size_t *length,
 
 	switch (vp->magic) {
 	case LLDP_SNMP_MED_LOCAL_CLASS:
-		long_ret = scfg->g_lchassis.c_med_type;
+		long_ret = LOCAL_CHASSIS(scfg)->c_med_type;
 		return (u_char *)&long_ret;
+	case LLDP_SNMP_MED_POE_DEVICETYPE:
+	case LLDP_SNMP_MED_POE_PSE_POWERSOURCE:
+	case LLDP_SNMP_MED_POE_PD_POWERVAL:
+	case LLDP_SNMP_MED_POE_PD_POWERSOURCE:
+	case LLDP_SNMP_MED_POE_PD_POWERPRIORITY:
+		/* LLDP-MED requires only one device type for all
+		   ports. Moreover, a PSE can only have one power source. At
+		   least, all PD values are global and not per-port. We try to
+		   do our best. For device type, we decide on the number of
+		   PD/PSE ports. */
+		pse = 0; power = NULL;
+		TAILQ_FOREACH(hardware, &scfg->g_hardware, h_entries) {
+			if (hardware->h_lport.p_med_power.devicetype ==
+			    LLDPMED_POW_TYPE_PSE) {
+				pse++;
+				if (pse == 1) /* Take this port as a reference */
+					power = &hardware->h_lport.p_med_power;
+			} else if (hardware->h_lport.p_med_power.devicetype ==
+			    LLDPMED_POW_TYPE_PD) {
+				pse--;
+				if (pse == -1) /* Take this one instead */
+					power = &hardware->h_lport.p_med_power;
+			}
+		}
+		if (!power)
+			break;	/* Neither PSE nor PD */
+		long_ret = agent_h_med_power(vp, power);
+		if (long_ret != (unsigned long)-1)
+			return (u_char *)&long_ret;
+		break;
 
 #define LLDP_H_LOCAL_MED(magic, variable)				\
 		case magic:						\
-		    if (scfg->g_lchassis.variable) {			\
+		    if (LOCAL_CHASSIS(scfg)->variable) {		\
 			    *var_len = strlen(				\
-				    scfg->g_lchassis.variable);		\
-			    return (u_char *)scfg->g_lchassis.variable;	\
+				    LOCAL_CHASSIS(scfg)->variable);		\
+			    return (u_char *)LOCAL_CHASSIS(scfg)->variable;	\
 		    }							\
 		break
 
@@ -513,19 +710,95 @@ agent_h_local_med(struct variable *vp, oid *name, size_t *length,
 	return NULL;
 }
 
+static u_char*
+agent_h_local_med_policy(struct variable *vp, oid *name, size_t *length,
+    int exact, size_t *var_len, WriteMethod **write_method)
+{
+	struct lldpd_med_policy *policy;
+        static unsigned long long_ret;
+
+	if ((policy = (struct lldpd_med_policy *)header_pmedindexed_table(vp, name, length,
+		    exact, var_len, write_method, PMED_VARIANT_POLICY)) == NULL)
+		return NULL;
+
+	switch (vp->magic) {
+	case LLDP_SNMP_MED_LOCAL_POLICY_VID:
+		long_ret = policy->vid;
+		break;
+	case LLDP_SNMP_MED_LOCAL_POLICY_PRIO:
+		long_ret = policy->priority;
+		break;
+	case LLDP_SNMP_MED_LOCAL_POLICY_DSCP:
+		long_ret = policy->dscp;
+		break;
+	case LLDP_SNMP_MED_LOCAL_POLICY_UNKNOWN:
+		long_ret = policy->unknown?1:2;
+		break;
+	case LLDP_SNMP_MED_LOCAL_POLICY_TAGGED:
+		long_ret = policy->tagged?1:2;
+		break;
+	default:
+		return NULL;
+	}
+	return (u_char *)&long_ret;
+}
+
+static u_char*
+agent_h_local_med_location(struct variable *vp, oid *name, size_t *length,
+    int exact, size_t *var_len, WriteMethod **write_method)
+{
+	struct lldpd_med_loc *location;
+
+	if ((location = (struct lldpd_med_loc *)header_pmedindexed_table(vp, name, length,
+		    exact, var_len, write_method, PMED_VARIANT_LOCATION)) == NULL)
+		return NULL;
+
+	switch (vp->magic) {
+	case LLDP_SNMP_MED_LOCAL_LOCATION:
+		*var_len = location->data_len;
+		return (u_char *)location->data;
+	}
+	return NULL;
+}
+
+static u_char*
+agent_h_local_med_power(struct variable *vp, oid *name, size_t *length,
+    int exact, size_t *var_len, WriteMethod **write_method)
+{
+	static unsigned long long_ret;
+	struct lldpd_hardware *hardware;
+
+	if ((hardware = header_portindexed_table(vp, name, length,
+		    exact, var_len, write_method)) == NULL)
+		return NULL;
+	if (!hardware->h_lport.p_med_power.devicetype)
+		goto localpower_failed;
+
+	long_ret = agent_h_med_power(vp, &hardware->h_lport.p_med_power);
+	if (long_ret != (unsigned long)-1)
+		return (u_char *)&long_ret;
+
+localpower_failed:
+	/* No valid data was found, we should try the next one! */
+	if (!exact && (name[*length-1] < MAX_SUBID))
+		return agent_h_local_med_power(vp, name, length,
+		    exact, var_len, write_method);
+        return NULL;
+}
+
 static u_char*
 agent_h_remote_med(struct variable *vp, oid *name, size_t *length,
     int exact, size_t *var_len, WriteMethod **write_method)
 {
-	struct lldpd_hardware *hardware;
+	struct lldpd_port *port;
 	static uint8_t bit;
         static unsigned long long_ret;
 
-	if ((hardware = header_tprindexed_table(vp, name, length,
+	if ((port = header_tprindexed_table(vp, name, length,
 		    exact, var_len, write_method, TPR_VARIANT_NONE)) == NULL)
 		return NULL;
 
-	if (!hardware->h_rchassis->c_med_cap_available) {
+	if (!port->p_chassis->c_med_cap_available) {
 		if (!exact && (name[*length-2] < MAX_SUBID))
 			name[*length-2]++;
 		goto remotemed_failed;
@@ -533,99 +806,36 @@ agent_h_remote_med(struct variable *vp, oid *name, size_t *length,
 
 	switch (vp->magic) {
         case LLDP_SNMP_MED_REMOTE_CLASS:
-                long_ret = hardware->h_rchassis->c_med_type;
+                long_ret = port->p_chassis->c_med_type;
 		return (u_char *)&long_ret;
 	case LLDP_SNMP_MED_REMOTE_CAP_AVAILABLE:
 		*var_len = 1;
-		bit = swap_bits(hardware->h_rchassis->c_med_cap_available);
+		bit = swap_bits(port->p_chassis->c_med_cap_available);
 		return (u_char *)&bit;
 	case LLDP_SNMP_MED_REMOTE_CAP_ENABLED:
 		*var_len = 1;
-		bit = swap_bits(hardware->h_rport->p_med_cap_enabled);
+		bit = swap_bits(port->p_med_cap_enabled);
 		return (u_char *)&bit;
-	case LLDP_SNMP_MED_REMOTE_POE_DEVICETYPE:
-		switch (hardware->h_rport->p_med_pow_devicetype) {
-		case LLDPMED_POW_TYPE_PSE:
-			long_ret = 2; break;
-		case LLDPMED_POW_TYPE_PD:
-			long_ret = 3; break;
-		case 0:
-			long_ret = 4; break;
-		default:
-			long_ret = 1;
-		}
-		return (u_char *)&long_ret;
-	case LLDP_SNMP_MED_REMOTE_POE_PSE_POWERVAL:
-	case LLDP_SNMP_MED_REMOTE_POE_PD_POWERVAL:
-		if (((vp->magic == LLDP_SNMP_MED_REMOTE_POE_PSE_POWERVAL) &&
-			(hardware->h_rport->p_med_pow_devicetype ==
-			LLDPMED_POW_TYPE_PSE)) ||
-		    ((vp->magic == LLDP_SNMP_MED_REMOTE_POE_PD_POWERVAL) &&
-			(hardware->h_rport->p_med_pow_devicetype ==
-			    LLDPMED_POW_TYPE_PD))) {
-			long_ret = hardware->h_rport->p_med_pow_val;
+
+	case LLDP_SNMP_MED_POE_DEVICETYPE:
+	case LLDP_SNMP_MED_POE_PSE_POWERVAL:
+	case LLDP_SNMP_MED_POE_PD_POWERVAL:
+	case LLDP_SNMP_MED_POE_PSE_POWERSOURCE:
+	case LLDP_SNMP_MED_POE_PD_POWERSOURCE:
+	case LLDP_SNMP_MED_POE_PSE_POWERPRIORITY:
+	case LLDP_SNMP_MED_POE_PD_POWERPRIORITY:
+		long_ret = agent_h_med_power(vp, &port->p_med_power);
+		if (long_ret != (unsigned long)-1)
 			return (u_char *)&long_ret;
-		}
-		break;
-	case LLDP_SNMP_MED_REMOTE_POE_PSE_POWERSOURCE:
-		if (hardware->h_rport->p_med_pow_devicetype ==
-		    LLDPMED_POW_TYPE_PSE) {
-			switch (hardware->h_rport->p_med_pow_source) {
-			case LLDPMED_POW_SOURCE_PRIMARY:
-				long_ret = 2; break;
-			case LLDPMED_POW_SOURCE_BACKUP:
-				long_ret = 3; break;
-			default:
-				long_ret = 1;
-			}
-			return (u_char *)&long_ret;
-		}
-		break;
-	case LLDP_SNMP_MED_REMOTE_POE_PD_POWERSOURCE:
-		if (hardware->h_rport->p_med_pow_devicetype ==
-		    LLDPMED_POW_TYPE_PD) {
-			switch (hardware->h_rport->p_med_pow_source) {
-			case LLDPMED_POW_SOURCE_PSE:
-				long_ret = 2; break;
-			case LLDPMED_POW_SOURCE_LOCAL:
-				long_ret = 3; break;
-			case LLDPMED_POW_SOURCE_BOTH:
-				long_ret = 4; break;
-			default:
-				long_ret = 1;
-			}
-			return (u_char *)&long_ret;
-		}
-		break;
-	case LLDP_SNMP_MED_REMOTE_POE_PSE_POWERPRIORITY:
-	case LLDP_SNMP_MED_REMOTE_POE_PD_POWERPRIORITY:
-		if (((vp->magic == LLDP_SNMP_MED_REMOTE_POE_PSE_POWERPRIORITY) &&
-			(hardware->h_rport->p_med_pow_devicetype ==
-			LLDPMED_POW_TYPE_PSE)) ||
-		    ((vp->magic == LLDP_SNMP_MED_REMOTE_POE_PD_POWERPRIORITY) &&
-			(hardware->h_rport->p_med_pow_devicetype ==
-			    LLDPMED_POW_TYPE_PD))) {
-			switch (hardware->h_rport->p_med_pow_priority) {
-			case LLDPMED_POW_PRIO_CRITICAL:
-				long_ret = 2; break;
-			case LLDPMED_POW_PRIO_HIGH:
-				long_ret = 3; break;
-			case LLDPMED_POW_PRIO_LOW:
-				long_ret = 4; break;
-			default:
-				long_ret = 1;
-			}
-			return (u_char *)&long_ret;
-		}
 		break;
 
 #define LLDP_H_REMOTE_MED(magic, variable)				\
 		case magic:						\
-		    if (hardware->h_rchassis->variable) {		\
+		    if (port->p_chassis->variable) {		\
 			    *var_len = strlen(				\
-				    hardware->h_rchassis->variable);	\
+				    port->p_chassis->variable);	\
 			    return (u_char *)				\
-				hardware->h_rchassis->variable;		\
+				port->p_chassis->variable;		\
 		    }							\
 		break
 
@@ -660,15 +870,15 @@ agent_h_remote_med_policy(struct variable *vp, oid *name, size_t *length,
     int exact, size_t *var_len, WriteMethod **write_method)
 {
 	int type;
-	struct lldpd_hardware *hardware;
+	struct lldpd_port *port;
 	struct lldpd_med_policy *policy;
         static unsigned long long_ret;
 
-	if ((hardware = header_tprindexed_table(vp, name, length,
+	if ((port = header_tprindexed_table(vp, name, length,
 		    exact, var_len, write_method, TPR_VARIANT_MED_POLICY)) == NULL)
 		return NULL;
 
-	if (!hardware->h_rchassis->c_med_cap_available) {
+	if (!port->p_chassis->c_med_cap_available) {
 		if (!exact && (name[*length-2] < MAX_SUBID))
 			name[*length-2]++;
 		goto remotemedpolicy_failed;
@@ -677,7 +887,7 @@ agent_h_remote_med_policy(struct variable *vp, oid *name, size_t *length,
 	type = name[*length - 1];
 	if ((type < 1) || (type > LLDPMED_APPTYPE_LAST))
 		goto remotemedpolicy_failed;
-	policy = &hardware->h_rport->p_med_policy[type-1];
+	policy = &port->p_med_policy[type-1];
 	if (policy->type != type)
 		goto remotemedpolicy_failed;
 
@@ -713,14 +923,14 @@ agent_h_remote_med_location(struct variable *vp, oid *name, size_t *length,
     int exact, size_t *var_len, WriteMethod **write_method)
 {
 	int type;
-	struct lldpd_hardware *hardware;
+	struct lldpd_port *port;
 	struct lldpd_med_loc *location;
 
-	if ((hardware = header_tprindexed_table(vp, name, length,
+	if ((port = header_tprindexed_table(vp, name, length,
 		    exact, var_len, write_method, TPR_VARIANT_MED_LOCATION)) == NULL)
 		return NULL;
 
-	if (!hardware->h_rchassis->c_med_cap_available) {
+	if (!port->p_chassis->c_med_cap_available) {
 		if (!exact && (name[*length-2] < MAX_SUBID))
 			name[*length-2]++;
 		goto remotemedlocation_failed;
@@ -729,7 +939,7 @@ agent_h_remote_med_location(struct variable *vp, oid *name, size_t *length,
 	type = name[*length - 1];
 	if ((type < 1) || (type > LLDPMED_APPTYPE_LAST))
 		goto remotemedlocation_failed;
-	location = &hardware->h_rport->p_med_location[type-1];
+	location = &port->p_med_location[type-1];
 	if (location->format != type)
 		goto remotemedlocation_failed;
 
@@ -761,24 +971,24 @@ agent_h_local_chassis(struct variable *vp, oid *name, size_t *length,
 
 	switch (vp->magic) {
 	case LLDP_SNMP_LOCAL_CIDSUBTYPE:
-                long_ret = scfg->g_lchassis.c_id_subtype;
+                long_ret = LOCAL_CHASSIS(scfg)->c_id_subtype;
 		return (u_char *)&long_ret;
 	case LLDP_SNMP_LOCAL_CID:
-		*var_len = scfg->g_lchassis.c_id_len;
-		return (u_char *)scfg->g_lchassis.c_id;
+		*var_len = LOCAL_CHASSIS(scfg)->c_id_len;
+		return (u_char *)LOCAL_CHASSIS(scfg)->c_id;
 	case LLDP_SNMP_LOCAL_SYSNAME:
-		*var_len = strlen(scfg->g_lchassis.c_name);
-		return (u_char *)scfg->g_lchassis.c_name;
+		*var_len = strlen(LOCAL_CHASSIS(scfg)->c_name);
+		return (u_char *)LOCAL_CHASSIS(scfg)->c_name;
 	case LLDP_SNMP_LOCAL_SYSDESCR:
-		*var_len = strlen(scfg->g_lchassis.c_descr);
-		return (u_char *)scfg->g_lchassis.c_descr;
+		*var_len = strlen(LOCAL_CHASSIS(scfg)->c_descr);
+		return (u_char *)LOCAL_CHASSIS(scfg)->c_descr;
 	case LLDP_SNMP_LOCAL_SYSCAP_SUP:
 		*var_len = 1;
-		bit = swap_bits(scfg->g_lchassis.c_cap_available);
+		bit = swap_bits(LOCAL_CHASSIS(scfg)->c_cap_available);
 		return (u_char *)&bit;
 	case LLDP_SNMP_LOCAL_SYSCAP_ENA:
 		*var_len = 1;
-		bit = swap_bits(scfg->g_lchassis.c_cap_enabled);
+		bit = swap_bits(LOCAL_CHASSIS(scfg)->c_cap_enabled);
 		return (u_char *)&bit;
 	default:
 		break;
@@ -930,71 +1140,71 @@ static u_char*
 agent_h_remote_port(struct variable *vp, oid *name, size_t *length,
     int exact, size_t *var_len, WriteMethod **write_method)
 {
-	struct lldpd_hardware *hardware;
+	struct lldpd_port *port;
 	static uint8_t bit;
         static unsigned long long_ret;
 
-	if ((hardware = header_tprindexed_table(vp, name, length,
+	if ((port = header_tprindexed_table(vp, name, length,
 		    exact, var_len, write_method, TPR_VARIANT_NONE)) == NULL)
 		return NULL;
 
 	switch (vp->magic) {
         case LLDP_SNMP_REMOTE_CIDSUBTYPE:
-                long_ret = hardware->h_rchassis->c_id_subtype;
+                long_ret = port->p_chassis->c_id_subtype;
                 return (u_char *)&long_ret;
         case LLDP_SNMP_REMOTE_CID:
-		*var_len = hardware->h_rchassis->c_id_len;
-		return (u_char *)hardware->h_rchassis->c_id;
+		*var_len = port->p_chassis->c_id_len;
+		return (u_char *)port->p_chassis->c_id;
         case LLDP_SNMP_REMOTE_PIDSUBTYPE:
-                long_ret = hardware->h_rport->p_id_subtype;
+                long_ret = port->p_id_subtype;
 		return (u_char *)&long_ret;
         case LLDP_SNMP_REMOTE_PID:
-		*var_len = hardware->h_rport->p_id_len;
-		return (u_char *)hardware->h_rport->p_id;
+		*var_len = port->p_id_len;
+		return (u_char *)port->p_id;
         case LLDP_SNMP_REMOTE_PORTDESC:
-		*var_len = strlen(hardware->h_rport->p_descr);
-		return (u_char *)hardware->h_rport->p_descr;
+		*var_len = strlen(port->p_descr);
+		return (u_char *)port->p_descr;
         case LLDP_SNMP_REMOTE_SYSNAME:
-		*var_len = strlen(hardware->h_rchassis->c_name);
-		return (u_char *)hardware->h_rchassis->c_name;
+		*var_len = strlen(port->p_chassis->c_name);
+		return (u_char *)port->p_chassis->c_name;
         case LLDP_SNMP_REMOTE_SYSDESC:
-		*var_len = strlen(hardware->h_rchassis->c_descr);
-		return (u_char *)hardware->h_rchassis->c_descr;
+		*var_len = strlen(port->p_chassis->c_descr);
+		return (u_char *)port->p_chassis->c_descr;
         case LLDP_SNMP_REMOTE_SYSCAP_SUP:
 		*var_len = 1;
-		bit = swap_bits(hardware->h_rchassis->c_cap_available);
+		bit = swap_bits(port->p_chassis->c_cap_available);
 		return (u_char *)&bit;
         case LLDP_SNMP_REMOTE_SYSCAP_ENA:
 		*var_len = 1;
-		bit = swap_bits(hardware->h_rchassis->c_cap_enabled);
+		bit = swap_bits(port->p_chassis->c_cap_enabled);
 		return (u_char *)&bit;
 #ifdef ENABLE_DOT3
         case LLDP_SNMP_REMOTE_DOT3_AUTONEG_SUPPORT:
-                long_ret = 2 - hardware->h_rport->p_autoneg_support;
+                long_ret = 2 - port->p_autoneg_support;
                 return (u_char *)&long_ret;
         case LLDP_SNMP_REMOTE_DOT3_AUTONEG_ENABLED:
-                long_ret = 2 - hardware->h_rport->p_autoneg_enabled;
+                long_ret = 2 - port->p_autoneg_enabled;
                 return (u_char *)&long_ret;
         case LLDP_SNMP_REMOTE_DOT3_AUTONEG_ADVERTISED:
                 *var_len = 2;
-                return (u_char *)&hardware->h_rport->p_autoneg_advertised;
+                return (u_char *)&port->p_autoneg_advertised;
         case LLDP_SNMP_REMOTE_DOT3_AUTONEG_MAU:
-                long_ret = hardware->h_rport->p_mau_type;
+                long_ret = port->p_mau_type;
                 return (u_char *)&long_ret;
         case LLDP_SNMP_REMOTE_DOT3_AGG_STATUS:
-                bit = swap_bits((hardware->h_rport->p_aggregid > 0) ? 3 : 0);
+                bit = swap_bits((port->p_aggregid > 0) ? 3 : 0);
                 *var_len = 1;
                 return (u_char *)&bit;
         case LLDP_SNMP_REMOTE_DOT3_AGG_ID:
-                long_ret = hardware->h_rport->p_aggregid;
+                long_ret = port->p_aggregid;
                 return (u_char *)&long_ret;
         case LLDP_SNMP_REMOTE_DOT3_MFS:
-                long_ret = hardware->h_rport->p_mfs;
+                long_ret = port->p_mfs;
                 return (u_char *)&long_ret;
 #endif
 #ifdef ENABLE_DOT1
         case LLDP_SNMP_REMOTE_DOT1_PVID:
-                long_ret = hardware->h_rport->p_pvid;
+                long_ret = port->p_pvid;
                 return (u_char *)&long_ret;
 #endif
 	default:
@@ -1041,7 +1251,7 @@ agent_h_local_management(struct variable *vp, oid *name, size_t *length,
         oid *target, best[6];
         int result, target_len;
 
-        if (scfg->g_lchassis.c_mgmt.s_addr == INADDR_ANY)
+        if (LOCAL_CHASSIS(scfg)->c_mgmt.s_addr == INADDR_ANY)
                 return NULL;
 
         if ((result = snmp_oid_compare(name, *length, vp->name, vp->namelen)) < 0) {
@@ -1057,10 +1267,10 @@ agent_h_local_management(struct variable *vp, oid *name, size_t *length,
 
         best[0] = 1;
         best[1] = 4;
-        best[5] = scfg->g_lchassis.c_mgmt.s_addr >> 24;
-        best[4] = (scfg->g_lchassis.c_mgmt.s_addr & 0xffffff) >> 16;
-        best[3] = (scfg->g_lchassis.c_mgmt.s_addr & 0xffff) >> 8;
-        best[2] = scfg->g_lchassis.c_mgmt.s_addr & 0xff;
+        best[5] = LOCAL_CHASSIS(scfg)->c_mgmt.s_addr >> 24;
+        best[4] = (LOCAL_CHASSIS(scfg)->c_mgmt.s_addr & 0xffffff) >> 16;
+        best[3] = (LOCAL_CHASSIS(scfg)->c_mgmt.s_addr & 0xffff) >> 8;
+        best[2] = LOCAL_CHASSIS(scfg)->c_mgmt.s_addr & 0xff;
 
         if ((result = snmp_oid_compare(target, target_len, best, 6)) < 0) {
                 if (exact)
@@ -1072,20 +1282,20 @@ agent_h_local_management(struct variable *vp, oid *name, size_t *length,
         else if (!exact && result == 0)
                 return NULL;
 
-        return agent_management(vp, var_len, &scfg->g_lchassis);
+        return agent_management(vp, var_len, LOCAL_CHASSIS(scfg));
 }
 
 static u_char*
 agent_h_remote_management(struct variable *vp, oid *name, size_t *length,
     int exact, size_t *var_len, WriteMethod **write_method)
 {
-	struct lldpd_hardware *hardware;
+	struct lldpd_port *port;
 
-	if ((hardware = header_tprindexed_table(vp, name, length,
+	if ((port = header_tprindexed_table(vp, name, length,
 		    exact, var_len, write_method, TPR_VARIANT_IP)) == NULL)
 		return NULL;
 
-        return agent_management(vp, var_len, hardware->h_rchassis);
+        return agent_management(vp, var_len, port->p_chassis);
 }
 
 static struct variable8 lldp_vars[] = {
@@ -1194,6 +1404,16 @@ static struct variable8 lldp_vars[] = {
 	/* LLDP-MED local */
 	{LLDP_SNMP_MED_LOCAL_CLASS, ASN_INTEGER, RONLY, agent_h_local_med, 6,
 	 {1, 5, 4795, 1, 1, 1}},
+	{LLDP_SNMP_MED_LOCAL_POLICY_VID, ASN_INTEGER, RONLY, agent_h_local_med_policy, 8,
+	 {1, 5, 4795, 1, 2, 1, 1, 2}},
+	{LLDP_SNMP_MED_LOCAL_POLICY_PRIO, ASN_INTEGER, RONLY, agent_h_local_med_policy, 8,
+	 {1, 5, 4795, 1, 2, 1, 1, 3}},
+	{LLDP_SNMP_MED_LOCAL_POLICY_DSCP, ASN_INTEGER, RONLY, agent_h_local_med_policy, 8,
+	 {1, 5, 4795, 1, 2, 1, 1, 4}},
+	{LLDP_SNMP_MED_LOCAL_POLICY_UNKNOWN, ASN_INTEGER, RONLY, agent_h_local_med_policy, 8,
+	 {1, 5, 4795, 1, 2, 1, 1, 5}},
+	{LLDP_SNMP_MED_LOCAL_POLICY_TAGGED, ASN_INTEGER, RONLY, agent_h_local_med_policy, 8,
+	 {1, 5, 4795, 1, 2, 1, 1, 6}},
 	{LLDP_SNMP_MED_LOCAL_HW, ASN_OCTET_STR, RONLY, agent_h_local_med, 6,
 	 {1, 5, 4795, 1, 2, 2}},
 	{LLDP_SNMP_MED_LOCAL_FW, ASN_OCTET_STR, RONLY, agent_h_local_med, 6,
@@ -1208,6 +1428,22 @@ static struct variable8 lldp_vars[] = {
 	 {1, 5, 4795, 1, 2, 7}},
 	{LLDP_SNMP_MED_LOCAL_ASSET, ASN_OCTET_STR, RONLY, agent_h_local_med, 6,
 	 {1, 5, 4795, 1, 2, 8}},
+	{LLDP_SNMP_MED_LOCAL_LOCATION, ASN_OCTET_STR, RONLY, agent_h_local_med_location, 8,
+	 {1, 5, 4795, 1, 2, 9, 1, 2}},
+	{LLDP_SNMP_MED_POE_DEVICETYPE, ASN_INTEGER, RONLY, agent_h_local_med, 6,
+	 {1, 5, 4795, 1, 2, 10}},
+	{LLDP_SNMP_MED_POE_PSE_POWERVAL, ASN_GAUGE, RONLY, agent_h_local_med_power, 8,
+	 {1, 5, 4795, 1, 2, 11, 1, 1}},
+	{LLDP_SNMP_MED_POE_PSE_POWERPRIORITY, ASN_INTEGER, RONLY, agent_h_local_med_power, 8,
+	 {1, 5, 4795, 1, 2, 11, 1, 2}},
+	{LLDP_SNMP_MED_POE_PSE_POWERSOURCE, ASN_INTEGER, RONLY, agent_h_local_med, 6,
+	 {1, 5, 4795, 1, 2, 12}},
+	{LLDP_SNMP_MED_POE_PD_POWERVAL, ASN_GAUGE, RONLY, agent_h_local_med, 6,
+	 {1, 5, 4795, 1, 2, 13}},
+	{LLDP_SNMP_MED_POE_PD_POWERSOURCE, ASN_INTEGER, RONLY, agent_h_local_med, 6,
+	 {1, 5, 4795, 1, 2, 14}},
+	{LLDP_SNMP_MED_POE_PD_POWERPRIORITY, ASN_INTEGER, RONLY, agent_h_local_med, 6,
+	 {1, 5, 4795, 1, 2, 15}},
 	/* LLDP-MED remote */
 	{LLDP_SNMP_MED_REMOTE_CAP_AVAILABLE, ASN_OCTET_STR, RONLY, agent_h_remote_med, 8,
 	 {1, 5, 4795, 1, 3, 1, 1, 1}},
@@ -1241,28 +1477,32 @@ static struct variable8 lldp_vars[] = {
 	 {1, 5, 4795, 1, 3, 2, 1, 6}},
 	{LLDP_SNMP_MED_REMOTE_LOCATION, ASN_OCTET_STR, RONLY, agent_h_remote_med_location, 8,
 	 {1, 5, 4795, 1, 3, 4, 1, 2}},
-	{LLDP_SNMP_MED_REMOTE_POE_DEVICETYPE, ASN_INTEGER, RONLY, agent_h_remote_med, 8,
+	{LLDP_SNMP_MED_POE_DEVICETYPE, ASN_INTEGER, RONLY, agent_h_remote_med, 8,
 	 {1, 5, 4795, 1, 3, 5, 1, 1}},
-	{LLDP_SNMP_MED_REMOTE_POE_PSE_POWERVAL, ASN_GAUGE, RONLY, agent_h_remote_med, 8,
+	{LLDP_SNMP_MED_POE_PSE_POWERVAL, ASN_GAUGE, RONLY, agent_h_remote_med, 8,
 	 {1, 5, 4795, 1, 3, 6, 1, 1}},
-	{LLDP_SNMP_MED_REMOTE_POE_PSE_POWERSOURCE, ASN_INTEGER, RONLY, agent_h_remote_med, 8,
+	{LLDP_SNMP_MED_POE_PSE_POWERSOURCE, ASN_INTEGER, RONLY, agent_h_remote_med, 8,
 	 {1, 5, 4795, 1, 3, 6, 1, 2}},
-	{LLDP_SNMP_MED_REMOTE_POE_PSE_POWERPRIORITY, ASN_INTEGER, RONLY, agent_h_remote_med, 8,
+	{LLDP_SNMP_MED_POE_PSE_POWERPRIORITY, ASN_INTEGER, RONLY, agent_h_remote_med, 8,
 	 {1, 5, 4795, 1, 3, 6, 1, 3}},
-	{LLDP_SNMP_MED_REMOTE_POE_PD_POWERVAL, ASN_GAUGE, RONLY, agent_h_remote_med, 8,
+	{LLDP_SNMP_MED_POE_PD_POWERVAL, ASN_GAUGE, RONLY, agent_h_remote_med, 8,
 	 {1, 5, 4795, 1, 3, 7, 1, 1}},
-	{LLDP_SNMP_MED_REMOTE_POE_PD_POWERSOURCE, ASN_INTEGER, RONLY, agent_h_remote_med, 8,
+	{LLDP_SNMP_MED_POE_PD_POWERSOURCE, ASN_INTEGER, RONLY, agent_h_remote_med, 8,
 	 {1, 5, 4795, 1, 3, 7, 1, 2}},
-	{LLDP_SNMP_MED_REMOTE_POE_PD_POWERPRIORITY, ASN_INTEGER, RONLY, agent_h_remote_med, 8,
+	{LLDP_SNMP_MED_POE_PD_POWERPRIORITY, ASN_INTEGER, RONLY, agent_h_remote_med, 8,
 	 {1, 5, 4795, 1, 3, 7, 1, 3}},
 #endif
 };
 
 void
-agent_init(struct lldpd *cfg, int debug)
+agent_init(struct lldpd *cfg, char *agentx, int debug)
 {
 	int rc;
+#ifdef HAVE___PROGNAME
 	extern char *__progname;
+#else
+#  define __progname "lldpd"
+#endif
 
 	LLOG_INFO("Enable SNMP subagent");
 	netsnmp_enable_subagent();
@@ -1283,6 +1523,9 @@ agent_init(struct lldpd *cfg, int debug)
 	/* We provide our UNIX domain transport */
 	agent_priv_register_domain();
 
+	if (agentx)
+		netsnmp_ds_set_string(NETSNMP_DS_APPLICATION_ID,
+				      NETSNMP_DS_AGENT_X_SOCKET, agentx);
 	init_agent("lldpAgent");
 	REGISTER_MIB("lldp", lldp_vars, variable8, lldp_oid);
 	init_snmp("lldpAgent");
