@@ -19,10 +19,13 @@
 static struct client_handle client_handles[] = {
 	{ HMSG_NONE, client_handle_none },
 	{ HMSG_GET_INTERFACES, client_handle_get_interfaces },
-	{ HMSG_GET_CHASSIS, client_handle_port_related },
+	{ HMSG_GET_NB_PORTS, client_handle_port_related },
 	{ HMSG_GET_PORT, client_handle_port_related },
+	{ HMSG_GET_CHASSIS, client_handle_port_related },
 #ifdef ENABLE_LLDPMED
 	{ HMSG_SET_LOCATION, client_handle_port_related },
+	{ HMSG_SET_POLICY, client_handle_port_related },
+	{ HMSG_SET_POWER, client_handle_port_related },
 #endif
 #ifdef ENABLE_DOT1
 	{ HMSG_GET_VLANS, client_handle_port_related },
@@ -31,7 +34,7 @@ static struct client_handle client_handles[] = {
 	{ 0, NULL } };
 
 void
-client_handle_client(struct lldpd *cfg, struct lldpd_client *client,
+client_handle_client(struct lldpd *cfg, struct lldpd_callback *callback,
     char *buffer, int n)
 {
 	struct hmsg *h;		/* Reception */
@@ -63,7 +66,7 @@ client_handle_client(struct lldpd *cfg, struct lldpd_client *client,
 				t->hdr.len = 0;
 				t->hdr.type = HMSG_NONE;
 			}
-			if (ctl_msg_send(client->fd, t) == -1)
+			if (ctl_msg_send(callback->fd, t) == -1)
 				LLOG_WARN("unable to send answer to client %d",
 				    h->hdr.pid);
 			free(t);
@@ -133,10 +136,9 @@ client_handle_port_related(struct lldpd *cfg, struct hmsg *r, struct hmsg *s)
 {
 	char *ifname;
 	struct lldpd_hardware *hardware;
+	struct lldpd_port *port;
 	void *p;
-#ifdef ENABLE_LLDPMED
 	int i;
-#endif
 
 	ifname = (char*)(&r->data);
 	if ((r->hdr.len < IFNAMSIZ) || (ifname[IFNAMSIZ - 1] != 0)) {
@@ -147,15 +149,6 @@ client_handle_port_related(struct lldpd *cfg, struct hmsg *r, struct hmsg *s)
 	}
 	TAILQ_FOREACH(hardware, &cfg->g_hardware, h_entries) {
 		if (strncmp(ifname, hardware->h_ifname, IFNAMSIZ) == 0) {
-			if (r->hdr.type != HMSG_SET_LOCATION) {
-				if ((hardware->h_rport == NULL) ||
-				    (hardware->h_rchassis == NULL)) {
-					s->hdr.len = 0;
-					s->hdr.type = HMSG_NONE;
-					return;
-				}
-			}
-			p = &s->data;
 			switch (r->hdr.type) {
 #ifdef ENABLE_LLDPMED
 			case HMSG_SET_LOCATION:
@@ -175,35 +168,134 @@ client_handle_port_related(struct lldpd *cfg, struct hmsg *r, struct hmsg *s)
 				}
 				hardware->h_lport.p_med_cap_enabled |= LLDPMED_CAP_LOCATION;
 				break;
+			case HMSG_SET_POLICY:
+				p = (char*)&r->data + IFNAMSIZ;
+				for (i=0; i < LLDPMED_APPTYPE_LAST; i++) {
+					hardware->h_lport.p_med_policy[i].type     = 0;
+					hardware->h_lport.p_med_policy[i].unknown  = 0;
+					hardware->h_lport.p_med_policy[i].tagged   = 0;
+					hardware->h_lport.p_med_policy[i].vid      = 0;
+					hardware->h_lport.p_med_policy[i].priority = 0;
+					hardware->h_lport.p_med_policy[i].dscp     = 0;
+				}
+				if (ctl_msg_unpack_structure(
+					STRUCT_LLDPD_MED_POLICY
+					STRUCT_LLDPD_MED_POLICY
+					STRUCT_LLDPD_MED_POLICY
+					STRUCT_LLDPD_MED_POLICY
+					STRUCT_LLDPD_MED_POLICY
+					STRUCT_LLDPD_MED_POLICY
+					STRUCT_LLDPD_MED_POLICY
+					STRUCT_LLDPD_MED_POLICY,
+					hardware->h_lport.p_med_policy,
+					8*sizeof(struct lldpd_med_policy),
+					r, &p) == -1) {
+					LLOG_WARNX("unable to set network policy for %s", ifname);
+					s->hdr.len = -1;
+					return;
+				}
+				hardware->h_lport.p_med_cap_enabled |=
+					LLDPMED_CAP_POLICY;
+				break;
+			case HMSG_SET_POWER:
+				p = (char*)&r->data + IFNAMSIZ;
+				memset(&hardware->h_lport.p_med_power, 0,
+				       sizeof(struct lldpd_med_power));
+				if (ctl_msg_unpack_structure(STRUCT_LLDPD_MED_POWER,
+						&hardware->h_lport.p_med_power,
+						sizeof(struct lldpd_med_power),
+						r, &p) == -1) {
+					LLOG_WARNX("unable to set POE-MDI for %s",
+						   ifname);
+					s->hdr.len = -1;
+					return;
+				}
+				hardware->h_lport.p_med_cap_enabled &= ~(
+					LLDPMED_CAP_MDI_PD | LLDPMED_CAP_MDI_PSE);
+				switch (hardware->h_lport.p_med_power.devicetype)
+				{
+				case LLDPMED_POW_TYPE_PSE:
+					hardware->h_lport.p_med_cap_enabled |=
+					    LLDPMED_CAP_MDI_PSE;
+					break;
+				case LLDPMED_POW_TYPE_PD:
+					hardware->h_lport.p_med_cap_enabled |=
+					    LLDPMED_CAP_MDI_PD;
+					break;
+				}
+				break;
 #endif
-#ifdef ENABLE_DOT1
+			case HMSG_GET_NB_PORTS:
+				p = &s->data;
+				i = 0;
+				TAILQ_FOREACH(port, &hardware->h_rports, p_entries) {
+					if (SMART_HIDDEN(cfg, port)) continue;
+					i++;
+				}
+				memcpy(p, &i, sizeof(int));
+				s->hdr.len = sizeof(int);
+				break;
 			case HMSG_GET_VLANS:
-				if (ctl_msg_pack_list(STRUCT_LLDPD_VLAN,
-					&hardware->h_rport->p_vlans,
-					sizeof(struct lldpd_vlan), s, &p) == -1) {
-					LLOG_WARNX("unable to send vlans information for "
-					    "interface %s for %d", ifname, r->hdr.pid);
-					s->hdr.len = -1;
-					return;
-				}
-				break;
-#endif
 			case HMSG_GET_PORT:
-				if (ctl_msg_pack_structure(STRUCT_LLDPD_PORT,
-					hardware->h_rport,
-					sizeof(struct lldpd_port), s, &p) == -1) {
-					LLOG_WARNX("unable to send port information for "
-					    "interface %s for %d", ifname, r->hdr.pid);
+			case HMSG_GET_CHASSIS:
+				/* We read the index which is right after the interface name */
+				if (r->hdr.len < IFNAMSIZ + sizeof(int)) {
+					LLOG_WARNX("too short message format for get "
+					    "port related message (%d)", r->hdr.type);
 					s->hdr.len = -1;
 					return;
 				}
-				break;
-			case HMSG_GET_CHASSIS:
-				if (ctl_msg_pack_structure(STRUCT_LLDPD_CHASSIS,
-					hardware->h_rchassis,
-					sizeof(struct lldpd_chassis), s, &p) == -1) {
-					LLOG_WARNX("unable to send chassis information for "
-					    "interface %s for %d", ifname, r->hdr.pid);
+				p = (char*)&r->data + IFNAMSIZ;
+				memcpy(&i, p, sizeof(int));
+				p = &s->data;
+				TAILQ_FOREACH(port, &hardware->h_rports, p_entries) {
+					if (SMART_HIDDEN(cfg, port)) continue;
+					if (i-- == 0) break;
+				}
+				if (!port) {
+					LLOG_INFO("out of range index requested for port "
+					    "related information on interface %s for %d",
+					    ifname, r->hdr.pid);
+					s->hdr.len = -1;
+					return;
+				}
+				p = (char*)&s->data;
+				switch (r->hdr.type) {
+#ifdef ENABLE_DOT1
+				case HMSG_GET_VLANS:
+					if (ctl_msg_pack_list(STRUCT_LLDPD_VLAN,
+						&port->p_vlans,
+						sizeof(struct lldpd_vlan), s, &p) == -1) {
+						LLOG_WARNX("unable to send vlans information for "
+						    "interface %s for %d", ifname, r->hdr.pid);
+						s->hdr.len = -1;
+						return;
+					}
+					break;
+#endif
+				case HMSG_GET_PORT:
+					if (ctl_msg_pack_structure(STRUCT_LLDPD_PORT,
+						port,
+						sizeof(struct lldpd_port), s, &p) == -1) {
+						LLOG_WARNX("unable to send port information for "
+						    "interface %s for %d", ifname, r->hdr.pid);
+						s->hdr.len = -1;
+						return;
+					}
+					break;
+				case HMSG_GET_CHASSIS:
+					if (ctl_msg_pack_structure(STRUCT_LLDPD_CHASSIS,
+						port->p_chassis,
+						sizeof(struct lldpd_chassis), s, &p) == -1) {
+						LLOG_WARNX("unable to send chassis information "
+						    "for interface %s for %d",
+						    ifname, r->hdr.pid);
+						s->hdr.len = -1;
+						return;
+					}
+					break;
+				default:
+					LLOG_WARNX("don't know what to do");
 					s->hdr.len = -1;
 					return;
 				}
