@@ -63,26 +63,6 @@ ctl_connect(char *name)
 	return s;
 }
 
-int
-ctl_accept(struct lldpd *cfg, int c)
-{
-	int s;
-	struct lldpd_client *lc;
-	if ((s = accept(c, NULL, NULL)) == -1) {
-		LLOG_WARN("unable to accept connection from socket");
-		return -1;
-	}
-	if ((lc = (struct lldpd_client *)malloc(sizeof(
-			    struct lldpd_client))) == NULL) {
-		LLOG_WARN("failed to allocate memory for new client");
-		close(s);
-		return -1;
-	}
-	lc->fd = s;
-	TAILQ_INSERT_TAIL(&cfg->g_clients, lc, next);
-	return 1;
-}
-
 void
 ctl_msg_init(struct hmsg *t, enum hmsg_type type)
 {
@@ -119,25 +99,6 @@ ctl_msg_recv(int fd, struct hmsg *t)
 	return 1;
 }
 
-int
-ctl_close(struct lldpd *cfg, int c)
-{
-	struct lldpd_client *client, *client_next;
-	for (client = TAILQ_FIRST(&cfg->g_clients);
-	    client != NULL;
-	    client = client_next) {
-		client_next = TAILQ_NEXT(client, next);
-		if (client->fd == c) {
-			close(client->fd);
-			TAILQ_REMOVE(&cfg->g_clients, client, next);
-			free(client);
-			return 1;
-		}
-	}
-	/* Not found */
-	return -1;
-}
-
 void
 ctl_cleanup(char *name)
 {
@@ -156,10 +117,12 @@ TAILQ_HEAD(gc_l, gc);
 
 typedef struct { char c; int16_t x; } st_int16;
 typedef struct { char c; int32_t x; } st_int32;
+typedef struct { char c; time_t x; } st_timet;
 typedef struct { char c; void *x; } st_void_p;
 
 #define INT16_ALIGN (sizeof(st_int16) - sizeof(int16_t))
 #define INT32_ALIGN (sizeof(st_int32) - sizeof(int32_t))
+#define TIMET_ALIGN (sizeof(st_timet) - sizeof(time_t))
 #define VOID_P_ALIGN (sizeof(st_void_p) - sizeof(void *))
 
 struct formatdef {
@@ -309,10 +272,10 @@ unpack_chars(struct hmsg *h, void **p, void *s,
     const struct formatdef *ct, struct gc_l *pointers)
 {
 	char *string;
-	struct {
+	struct __attribute__ ((__packed__)) {
 		char *string;
 		int len;
-	} reals __attribute__ ((__packed__));
+	} reals;
 	int len;
 	memcpy(&len, *p, sizeof(int));
 	*p += sizeof(int);
@@ -353,6 +316,8 @@ static struct formatdef conv_table[] = {
 	 pack_copy,	unpack_copy},
 	{'l',	4,				INT32_ALIGN,
 	 pack_copy,	unpack_copy},
+	{'t',	sizeof(time_t),			TIMET_ALIGN,
+	 pack_copy,	unpack_copy},
 	/* Null terminated string */
 	{'s',	sizeof(void*),			VOID_P_ALIGN,
 	 pack_string,	unpack_string},
@@ -365,7 +330,7 @@ static struct formatdef conv_table[] = {
 	/* Non null terminated string, followed by an int for the size */
 	{'C',	sizeof(void*) + sizeof(int),	VOID_P_ALIGN,
 	 pack_chars,	unpack_chars},
-	{0}
+	{0, 0, 0, NULL, NULL}
 };
 
 /* Lists can be packed only if the "next" member is the first one of the
