@@ -27,10 +27,11 @@
 #include <linux/sockios.h>
 
 int
-lldp_send(struct lldpd *global, struct lldpd_chassis *chassis,
+lldp_send(struct lldpd *global,
 	  struct lldpd_hardware *hardware)
 {
 	struct lldpd_port *port;
+	struct lldpd_chassis *chassis;
 	struct lldpd_frame *frame;
 	int length;
 	u_int8_t *packet, *pos, *tlv;
@@ -49,6 +50,7 @@ lldp_send(struct lldpd *global, struct lldpd_chassis *chassis,
 #endif
 
 	port = &hardware->h_lport;
+	chassis = port->p_chassis;
 	length = hardware->h_mtu;
 	if ((packet = (u_int8_t*)malloc(length)) == NULL)
 		return ENOMEM;
@@ -254,6 +256,65 @@ lldp_send(struct lldpd *global, struct lldpd_chassis *chassis,
 					goto toobig;
 			}
 		}
+
+		/* LLDP-MED network policy */
+		for (i = 0; i < LLDPMED_APPTYPE_LAST; i++) {
+			if (port->p_med_policy[i].type == i + 1) {
+				if (!(
+				      POKE_START_LLDP_TLV(LLDP_TLV_ORG) &&
+				      POKE_BYTES(med, sizeof(med)) &&
+				      POKE_UINT8(LLDP_TLV_MED_POLICY) &&
+				      POKE_UINT32((
+					((port->p_med_policy[i].type     %(1<< 8))<<24) |
+					((port->p_med_policy[i].unknown  %(1<< 1))<<23) |
+					((port->p_med_policy[i].tagged   %(1<< 1))<<22) |
+				      /*((0                              %(1<< 1))<<21) |*/
+					((port->p_med_policy[i].vid      %(1<<12))<< 9) |
+					((port->p_med_policy[i].priority %(1<< 3))<< 6) |
+					((port->p_med_policy[i].dscp     %(1<< 6))<< 0) )) &&
+				      POKE_END_LLDP_TLV))
+					goto toobig;
+			}
+		}
+
+		/* LLDP-MED POE-MDI */
+		if ((port->p_med_power.devicetype == LLDPMED_POW_TYPE_PSE) ||
+		    (port->p_med_power.devicetype == LLDPMED_POW_TYPE_PD)) {
+			int devicetype = 0, source = 0;
+			if (!(
+			      POKE_START_LLDP_TLV(LLDP_TLV_ORG) &&
+			      POKE_BYTES(med, sizeof(med)) &&
+			      POKE_UINT8(LLDP_TLV_MED_MDI)))
+				goto toobig;
+			switch (port->p_med_power.devicetype) {
+			case LLDPMED_POW_TYPE_PSE:
+				devicetype = 0;
+				switch (port->p_med_power.source) {
+				case LLDPMED_POW_SOURCE_PRIMARY: source = 1; break;
+				case LLDPMED_POW_SOURCE_BACKUP: source = 2; break;
+				case LLDPMED_POW_SOURCE_RESERVED: source = 3; break;
+				default: source = 0; break;
+				}
+				break;
+			case LLDPMED_POW_TYPE_PD:
+				devicetype = 1;
+				switch (port->p_med_power.source) {
+				case LLDPMED_POW_SOURCE_PSE: source = 1; break;
+				case LLDPMED_POW_SOURCE_LOCAL: source = 2; break;
+				case LLDPMED_POW_SOURCE_BOTH: source = 3; break;
+				default: source = 0; break;
+				}
+				break;
+			}
+			if (!(
+			      POKE_UINT8((
+				((devicetype                   %(1<< 2))<<6) |
+				((source                       %(1<< 2))<<4) |
+				((port->p_med_power.priority   %(1<< 4))<<0) )) &&
+			      POKE_UINT16(port->p_med_power.val) &&
+			      POKE_END_LLDP_TLV))
+				goto toobig;
+		}
 	}
 #endif
 
@@ -263,34 +324,28 @@ lldp_send(struct lldpd *global, struct lldpd_chassis *chassis,
 	      POKE_END_LLDP_TLV))
 		goto toobig;
 
-	if (!global->g_multi ||
-	    (hardware->h_mode == LLDPD_MODE_ANY) ||
-	    (hardware->h_mode == LLDPD_MODE_LLDP)) {
-		
-		if (write((hardware->h_raw_real > 0) ? hardware->h_raw_real :
-			hardware->h_raw, packet, 
-			pos - packet) == -1) {
-			LLOG_WARN("unable to send packet on real device for %s",
-			    hardware->h_ifname);
-			free(packet);
-			return ENETDOWN;
-		}
-
-		hardware->h_tx_cnt++;
+	if (hardware->h_ops->send(global, hardware,
+		(char *)packet, pos - packet) == -1) {
+		LLOG_WARN("unable to send packet on real device for %s",
+		    hardware->h_ifname);
+		free(packet);
+		return ENETDOWN;
 	}
 
+	hardware->h_tx_cnt++;
+
 	/* We assume that LLDP frame is the reference */
 	if ((frame = (struct lldpd_frame*)malloc(
 			sizeof(int) + pos - packet)) != NULL) {
 		frame->size = pos - packet;
 		memcpy(&frame->frame, packet, frame->size);
-		if ((hardware->h_llastframe == NULL) ||
-		    (hardware->h_llastframe->size != frame->size) ||
-		    (memcmp(hardware->h_llastframe->frame, frame->frame,
+		if ((hardware->h_lport.p_lastframe == NULL) ||
+		    (hardware->h_lport.p_lastframe->size != frame->size) ||
+		    (memcmp(hardware->h_lport.p_lastframe->frame, frame->frame,
 			frame->size) != 0)) {
-			free(hardware->h_llastframe);
-		hardware->h_llastframe = frame;
-		hardware->h_llastchange = time(NULL);
+			free(hardware->h_lport.p_lastframe);
+		hardware->h_lport.p_lastframe = frame;
+		hardware->h_lport.p_lastchange = time(NULL);
 		} else
 			free(frame);
 	}
@@ -607,75 +662,61 @@ lldp_decode(struct lldpd *cfg, char *frame, int s,
 					power = PEEK_UINT8;
 					switch (power & 0xC0) {
 					case 0x0:
-						port->p_med_pow_devicetype = LLDPMED_POW_TYPE_PSE;
+						port->p_med_power.devicetype = LLDPMED_POW_TYPE_PSE;
 						port->p_med_cap_enabled |=
 						    LLDPMED_CAP_MDI_PSE;
 						switch (power & 0x30) {
 						case 0x0:
-							port->p_med_pow_source =
+							port->p_med_power.source =
 							    LLDPMED_POW_SOURCE_UNKNOWN;
 							break;
 						case 0x10:
-							port->p_med_pow_source =
+							port->p_med_power.source =
 							    LLDPMED_POW_SOURCE_PRIMARY;
 							break;
 						case 0x20:
-							port->p_med_pow_source =
+							port->p_med_power.source =
 							    LLDPMED_POW_SOURCE_BACKUP;
 							break;
 						default:
-							port->p_med_pow_source =
+							port->p_med_power.source =
 							    LLDPMED_POW_SOURCE_RESERVED;
 						}
 						break;
 					case 0x40:
-						port->p_med_pow_devicetype = LLDPMED_POW_TYPE_PD;
+						port->p_med_power.devicetype = LLDPMED_POW_TYPE_PD;
 						port->p_med_cap_enabled |=
 						    LLDPMED_CAP_MDI_PD;
 						switch (power & 0x30) {
 						case 0x0:
-							port->p_med_pow_source =
+							port->p_med_power.source =
 							    LLDPMED_POW_SOURCE_UNKNOWN;
 							break;
 						case 0x10:
-							port->p_med_pow_source =
+							port->p_med_power.source =
 							    LLDPMED_POW_SOURCE_PSE;
 							break;
 						case 0x20:
-							port->p_med_pow_source =
+							port->p_med_power.source =
 							    LLDPMED_POW_SOURCE_LOCAL;
 							break;
 						default:
-							port->p_med_pow_source =
+							port->p_med_power.source =
 							    LLDPMED_POW_SOURCE_BOTH;
 						}
 						break;
 					default:
-						port->p_med_pow_devicetype =
+						port->p_med_power.devicetype =
 						    LLDPMED_POW_TYPE_RESERVED;
 					}
-					switch (power & 0x0F) {
-					case 0x0:
-						port->p_med_pow_priority =
+					if (((power & 0x0F) < 0) ||
+					    ((power & 0x0F) > LLDPMED_POW_PRIO_LOW))
+						port->p_med_power.priority =
 						    LLDPMED_POW_PRIO_UNKNOWN;
-						break;
-					case 0x1:
-						port->p_med_pow_priority =
-						    LLDPMED_POW_PRIO_CRITICAL;
-						break;
-					case 0x2:
-						port->p_med_pow_priority =
-						    LLDPMED_POW_PRIO_HIGH;
-						break;
-					case 0x3:
-						port->p_med_pow_priority =
-						    LLDPMED_POW_PRIO_LOW;
-						break;
-					default:
-						port->p_med_pow_priority =
-						    LLDPMED_POW_PRIO_UNKNOWN;
-					}
-					port->p_med_pow_val = PEEK_UINT16;
+					else
+						port->p_med_power.priority =
+						    power & 0x0F;
+					port->p_med_power.val = PEEK_UINT16;
 					break;
 				case LLDP_TLV_MED_IV_HW:
 				case LLDP_TLV_MED_IV_SW:
@@ -787,7 +828,8 @@ lldp_decode(struct lldpd *cfg, char *frame, int s,
 	*newport = port;
 	return 1;
 malformed:
-	lldpd_chassis_cleanup(chassis);
-	lldpd_port_cleanup(port, 1);
+	lldpd_chassis_cleanup(chassis, 1);
+	lldpd_port_cleanup(cfg, port, 1);
+	free(port);
 	return -1;
 }
