@@ -24,10 +24,8 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <errno.h>
-#include <sys/types.h>
 #include <sys/wait.h>
 #include <sys/stat.h>
-#include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/un.h>
 #include <regex.h>
@@ -39,8 +37,20 @@
 #include <netdb.h>
 #include <linux/sockios.h>
 #include <linux/if_packet.h>
-#include <netinet/in.h>
-#include <arpa/nameser.h>
+
+/* Use resolv.h */
+#ifdef HAVE_SYS_TYPES_H
+#  include <sys/types.h>
+#endif
+#ifdef HAVE_NETINET_IN_H
+#  include <netinet/in.h>
+#endif
+#ifdef HAVE_ARPA_NAMESER_H
+#  include <arpa/nameser.h> /* DNS HEADER struct */
+#endif
+#ifdef HAVE_NETDB_H
+#  include <netdb.h>
+#endif
 #include <resolv.h>
 
 enum {
@@ -57,7 +67,7 @@ enum {
 
 static int may_read(int, void *, size_t);
 static void must_read(int, void *, size_t);
-static void must_write(int, void *, size_t);
+static void must_write(int, const void *, size_t);
 
 static int remote;			/* Other side */
 static int monitored = -1;		/* Child */
@@ -152,22 +162,19 @@ priv_ethtool(char *ifname, struct ethtool_cmd *ethc)
 }
 
 int
-priv_iface_init(struct lldpd_hardware *hardware, int master)
+priv_iface_init(const char *name)
 {
 	int cmd, rc;
 	cmd = PRIV_IFACE_INIT;
 	must_write(remote, &cmd, sizeof(int));
-	must_write(remote, &master, sizeof(int));
-	must_write(remote, hardware->h_ifname, IFNAMSIZ);
+	must_write(remote, name, IFNAMSIZ);
 	must_read(remote, &rc, sizeof(int));
-	if (rc != 0)
-		return rc;	/* It's errno */
-	hardware->h_raw = receive_fd(remote);
-	return 0;
+	if (rc != 0) return -1;
+	return receive_fd(remote);
 }
 
 int
-priv_iface_multicast(char *name, u_int8_t *mac, int add)
+priv_iface_multicast(const char *name, u_int8_t *mac, int add)
 {
 	int cmd, rc;
 	cmd = PRIV_IFACE_MULTICAST;
@@ -260,6 +267,7 @@ asroot_open()
 		SYSFS_CLASS_NET "[^.][^/]*/brforward",
 		SYSFS_CLASS_NET "[^.][^/]*/brport",
 		SYSFS_CLASS_NET "[^.][^/]*/brif/[^.][^/]*/port_no",
+		SYSFS_CLASS_NET "[^.][^/]*/ifalias",
 		SYSFS_CLASS_DMI "product_version",
 		SYSFS_CLASS_DMI "product_serial",
 		SYSFS_CLASS_DMI "product_name",
@@ -268,7 +276,7 @@ asroot_open()
 		SYSFS_CLASS_DMI "chassis_asset_tag",
 		NULL
 	};
-	char **f;
+	const char **f;
 	char *file;
 	int fd, len, rc;
 	regex_t preg;
@@ -328,8 +336,6 @@ asroot_ethtool()
 	ifr.ifr_data = (caddr_t)&ethc;
 	ethc.cmd = ETHTOOL_GSET;
 	if ((rc = ioctl(sock, SIOCETHTOOL, &ifr)) != 0) {
-		LLOG_DEBUG("[priv]: unable to ioctl ETHTOOL for %s: %m",
-		    ifr.ifr_name);
 		must_write(remote, &rc, sizeof(int));
 		return;
 	}
@@ -341,11 +347,9 @@ static void
 asroot_iface_init()
 {
 	struct sockaddr_ll sa;
-	int un = 1;
-	int s, master;
+	int s;
 	char ifname[IFNAMSIZ];
 
-	must_read(remote, &master, sizeof(int));
 	must_read(remote, ifname, IFNAMSIZ);
 	ifname[IFNAMSIZ-1] = '\0';
 
@@ -358,27 +362,12 @@ asroot_iface_init()
 	memset(&sa, 0, sizeof(sa));
 	sa.sll_family = AF_PACKET;
 	sa.sll_protocol = 0;
-	if (master == -1)
-		sa.sll_ifindex = if_nametoindex(ifname);
-	else
-		sa.sll_ifindex = master;
+	sa.sll_ifindex = if_nametoindex(ifname);
 	if (bind(s, (struct sockaddr*)&sa, sizeof(sa)) < 0) {
 		must_write(remote, &errno, sizeof(errno));
 		close(s);
 		return;
 	}
-
-	if (master != -1) {
-		/* With bonding, we need to listen to bond device. We use
-		 * setsockopt() PACKET_ORIGDEV to get physical device instead of
-		 * bond device */
-		if (setsockopt(s, SOL_PACKET,
-			PACKET_ORIGDEV, &un, sizeof(un)) == -1) {
-			LLOG_WARN("[priv]: unable to setsockopt for master bonding device of %s. "
-                            "You will get inaccurate results",
-			    ifname);
-                }
-	}
 	errno = 0;
 	must_write(remote, &errno, sizeof(errno));
 	send_fd(remote, s);
@@ -644,9 +633,9 @@ must_read(int fd, void *buf, size_t n)
 /* Write data with the assertion that it all has to be written, or
  * else abort the process.  Based on atomicio() from openssh. */
 static void
-must_write(int fd, void *buf, size_t n)
+must_write(int fd, const void *buf, size_t n)
 {
-	char *s = buf;
+	const char *s = buf;
 	ssize_t res, pos = 0;
 
 	while (n > pos) {
