@@ -180,14 +180,16 @@ static struct sonmp_chassis sonmp_chassis_types[] = {
 };
 
 int
-sonmp_send(struct lldpd *global, struct lldpd_chassis *chassis,
+sonmp_send(struct lldpd *global,
     struct lldpd_hardware *hardware)
 {
 	const u_int8_t mcastaddr[] = SONMP_MULTICAST_ADDR;
 	const u_int8_t llcorg[] = LLC_ORG_NORTEL;
+	struct lldpd_chassis *chassis;
 	u_int8_t *packet, *pos, *pos_pid, *end;
 	int length;
 
+	chassis = hardware->h_lport.p_chassis;
 	length = hardware->h_mtu;
 	if ((packet = (u_int8_t*)malloc(length)) == NULL)
 		return ENOMEM;
@@ -224,7 +226,7 @@ sonmp_send(struct lldpd *global, struct lldpd_chassis *chassis,
 	      /* Segment on three bytes, we don't have slots, so we
 		 skip the first two bytes */
 	      POKE_UINT16(0) &&
-	      POKE_UINT8(if_nametoindex(hardware->h_ifname)) &&
+	      POKE_UINT8(hardware->h_ifindex) &&
 	      POKE_UINT8(1) &&  /* Chassis: Other */
 	      POKE_UINT8(12) &&	/* Back: Ethernet, Fast Ethernet and Gigabit */
 	      POKE_UINT8(SONMP_TOPOLOGY_NEW) && /* Should work. We have no state */
@@ -232,8 +234,8 @@ sonmp_send(struct lldpd *global, struct lldpd_chassis *chassis,
 	      POKE_SAVE(end)))
 		goto toobig;
 
-	if (write((hardware->h_raw_real > 0) ? hardware->h_raw_real :
-		hardware->h_raw, packet, end - packet) == -1) {
+	if (hardware->h_ops->send(global, hardware,
+		(char *)packet, end - packet) == -1) {
 		LLOG_WARN("unable to send packet on real device for %s",
 			   hardware->h_ifname);
 		free(packet);
@@ -246,8 +248,8 @@ sonmp_send(struct lldpd *global, struct lldpd_chassis *chassis,
 	PEEK_DISCARD(ETH_ALEN - 1); /* Modify the last byte of the MAC address */
 	POKE_UINT8(1);
 
-	if (write((hardware->h_raw_real > 0) ? hardware->h_raw_real :
-		hardware->h_raw, packet, end - packet) == -1) {
+	if (hardware->h_ops->send(global, hardware,
+		(char *)packet, end - packet) == -1) {
 		LLOG_WARN("unable to send second SONMP packet on real device for %s",
 			   hardware->h_ifname);
 		free(packet);
@@ -374,8 +376,9 @@ sonmp_decode(struct lldpd *cfg, char *frame, int s,
 	return 1;
 
 malformed:
-	lldpd_chassis_cleanup(chassis);
-	lldpd_port_cleanup(port, 1);
+	lldpd_chassis_cleanup(chassis, 1);
+	lldpd_port_cleanup(cfg, port, 1);
+	free(port);
 	return -1;
 }
 
